@@ -5,73 +5,30 @@
 # See LICENSE.txt for permissions.
 #++
 
-require 'rubygems/defaults'
-require 'thread'
-require 'etc'
-
 module Gem
-
-  RubyGemsVersion = VERSION = '1.3.5'
-
-  ##
-  # Raised when RubyGems is unable to load or activate a gem.  Contains the
-  # name and version requirements of the gem that either conflicts with
-  # already activated gems or that RubyGems is otherwise unable to activate.
-
-  class LoadError < ::LoadError
-
-    ##
-    # Name of gem
-
-    attr_accessor :name
-
-    ##
-    # Version requirement of gem
-
-    attr_accessor :version_requirement
-
-  end
-
+  QUICKLOADER_SUCKAGE = RUBY_VERSION =~ /^1\.9\.1/
+  GEM_PRELUDE_SUCKAGE = RUBY_VERSION =~ /^1\.9\.2/
 end
 
-module Kernel
+if Gem::GEM_PRELUDE_SUCKAGE and defined?(Gem::QuickLoader) then
+  Gem::QuickLoader.remove
 
-  ##
-  # Use Kernel#gem to activate a specific version of +gem_name+.
-  #
-  # +version_requirements+ is a list of version requirements that the
-  # specified gem must match, most commonly "= example.version.number".  See
-  # Gem::Requirement for how to specify a version requirement.
-  #
-  # If you will be activating the latest version of a gem, there is no need to
-  # call Kernel#gem, Kernel#require will do the right thing for you.
-  #
-  # Kernel#gem returns true if the gem was activated, otherwise false.  If the
-  # gem could not be found, didn't match the version requirements, or a
-  # different version was already activated, an exception will be raised.
-  #
-  # Kernel#gem should be called *before* any require statements (otherwise
-  # RubyGems may load a conflicting library version).
-  #
-  # In older RubyGems versions, the environment variable GEM_SKIP could be
-  # used to skip activation of specified gems, for example to test out changes
-  # that haven't been installed yet.  Now RubyGems defers to -I and the
-  # RUBYLIB environment variable to skip activation of a gem.
-  #
-  # Example:
-  #
-  #   GEM_SKIP=libA:libB ruby -I../libA -I../libB ./mycode.rb
+  $LOADED_FEATURES.delete Gem::QuickLoader.path_to_full_rubygems_library
 
-  def gem(gem_name, *version_requirements) # :doc:
-    skip_list = (ENV['GEM_SKIP'] || "").split(/:/)
-    raise Gem::LoadError, "skipping #{gem_name}" if skip_list.include? gem_name
-    Gem.activate(gem_name, *version_requirements)
+  if $LOADED_FEATURES.any? do |path| path.end_with? '/rubygems.rb' end then
+    # TODO path does not exist here
+    raise LoadError, "another rubygems is already loaded from #{path}"
   end
 
-  private :gem
-
+  class << Gem
+    remove_method :try_activate if Gem.respond_to?(:try_activate, true)
+  end
 end
 
+require 'rubygems/defaults'
+require "rubygems/dependency_list"
+require 'rbconfig'
+
 ##
 # RubyGems is the Ruby standard for publishing and managing third party
 # libraries.
@@ -86,6 +43,7 @@ end
 #
 # * {Creating Gems}[http://docs.rubygems.org/read/chapter/5]
 # * Gem::Specification
+# * Gem::Version for version dependency notes
 #
 # Further RubyGems documentation can be found at:
 #
@@ -96,7 +54,8 @@ end
 # == RubyGems Plugins
 #
 # As of RubyGems 1.3.2, RubyGems will load plugins installed in gems or
-# $LOAD_PATH.  Plugins must be named 'rubygems_plugin' are discovered via
+# $LOAD_PATH.  Plugins must be named 'rubygems_plugin' (.rb, .so, etc) and
+# placed at the root of your gem's #require_path.  Plugins are discovered via
 # Gem::find_files then loaded.  Take care when implementing a plugin as your
 # plugin file may be loaded multiple times if multiple versions of your gem
 # are installed.
@@ -122,7 +81,7 @@ end
 # == Bugs
 #
 # You can submit bugs to the
-# {RubyGems bug tracker}[http://rubyforge.org/tracker/?atid=575&group_id=126&func=browse]
+# {RubyGems bug tracker}[http://rubyforge.org/tracker/?atid=575&group_id=126]
 # on RubyForge
 #
 # == Credits
@@ -131,26 +90,26 @@ end
 #
 # RubyGems was originally developed at RubyConf 2003 by:
 #
-# * Rich Kilmer -- rich(at)infoether.com
-# * Chad Fowler -- chad(at)chadfowler.com
-# * David Black -- dblack(at)wobblini.net
+# * Rich Kilmer  -- rich(at)infoether.com
+# * Chad Fowler  -- chad(at)chadfowler.com
+# * David Black  -- dblack(at)wobblini.net
 # * Paul Brannan -- paul(at)atdesk.com
-# * Jim Weirch -- {jim(at)weirichhouse.org}[mailto:jim@weirichhouse.org]
+# * Jim Weirch   -- jim(at)weirichhouse.org
 #
 # Contributors:
 #
-# * Gavin Sinclair -- gsinclair(at)soyabean.com.au
-# * George Marrows -- george.marrows(at)ntlworld.com
-# * Dick Davies -- rasputnik(at)hellooperator.net
+# * Gavin Sinclair     -- gsinclair(at)soyabean.com.au
+# * George Marrows     -- george.marrows(at)ntlworld.com
+# * Dick Davies        -- rasputnik(at)hellooperator.net
 # * Mauricio Fernandez -- batsman.geo(at)yahoo.com
-# * Simon Strandgaard -- neoneye(at)adslhome.dk
-# * Dave Glasser -- glasser(at)mit.edu
-# * Paul Duncan -- pabs(at)pablotron.org
-# * Ville Aine -- vaine(at)cs.helsinki.fi
-# * Eric Hodel -- drbrain(at)segment7.net
-# * Daniel Berger -- djberg96(at)gmail.com
-# * Phil Hagelberg -- technomancy(at)gmail.com
-# * Ryan Davis
+# * Simon Strandgaard  -- neoneye(at)adslhome.dk
+# * Dave Glasser       -- glasser(at)mit.edu
+# * Paul Duncan        -- pabs(at)pablotron.org
+# * Ville Aine         -- vaine(at)cs.helsinki.fi
+# * Eric Hodel         -- drbrain(at)segment7.net
+# * Daniel Berger      -- djberg96(at)gmail.com
+# * Phil Hagelberg     -- technomancy(at)gmail.com
+# * Ryan Davis         -- ryand-ruby(at)zenspider.com
 #
 # (If your name is missing, PLEASE let us know!)
 #
@@ -159,41 +118,51 @@ end
 # -The RubyGems Team
 
 module Gem
+  VERSION = '1.6.2'
 
   ##
-  # Configuration settings from ::RbConfig
+  # Raised when RubyGems is unable to load or activate a gem.  Contains the
+  # name and version requirements of the gem that either conflicts with
+  # already activated gems or that RubyGems is otherwise unable to activate.
 
-  ConfigMap = {} unless defined?(ConfigMap)
+  class LoadError < ::LoadError
+    # Name of gem
+    attr_accessor :name
+
+    # Version requirement of gem
+    attr_accessor :requirement
+  end
 
-  require 'rbconfig'
   # :stopdoc:
-  RbConfig = Config unless defined? ::RbConfig
-  # :startdoc:
 
-  ConfigMap.merge!(
-    :EXEEXT => RbConfig::CONFIG["EXEEXT"],
-    :RUBY_SO_NAME => RbConfig::CONFIG["RUBY_SO_NAME"],
-    :arch => RbConfig::CONFIG["arch"],
-    :bindir => RbConfig::CONFIG["bindir"],
-    :datadir => RbConfig::CONFIG["datadir"],
-    :libdir => RbConfig::CONFIG["libdir"],
-    :ruby_install_name => RbConfig::CONFIG["ruby_install_name"],
-    :ruby_version => RbConfig::CONFIG["ruby_version"],
-    :sitedir => RbConfig::CONFIG["sitedir"],
-    :sitelibdir => RbConfig::CONFIG["sitelibdir"],
-    :vendordir => RbConfig::CONFIG["vendordir"] ,
-    :vendorlibdir => RbConfig::CONFIG["vendorlibdir"]
-  )
+  RubyGemsVersion = VERSION
+
+  RbConfigPriorities = %w[
+    EXEEXT RUBY_SO_NAME arch bindir datadir libdir ruby_install_name
+    ruby_version rubylibprefix sitedir sitelibdir vendordir vendorlibdir
+  ]
+
+  unless defined?(ConfigMap)
+    ##
+    # Configuration settings from ::RbConfig
+    ConfigMap = Hash.new do |cm, key|
+      cm[key] = RbConfig::CONFIG[key.to_s]
+    end
+  else
+    RbConfigPriorities.each do |key|
+      ConfigMap[key.to_sym] = RbConfig::CONFIG[key]
+    end
+  end
 
   ##
   # Default directories in a gem repository
 
   DIRECTORIES = %w[cache doc gems specifications] unless defined?(DIRECTORIES)
 
-  # :stopdoc:
-  MUTEX = Mutex.new
+  RubyGemsPackageVersion = VERSION
+
+  RUBYGEMS_DIR = File.dirname File.expand_path(__FILE__)
 
-  RubyGemsPackageVersion = RubyGemsVersion
   # :startdoc:
 
   ##
@@ -218,19 +187,42 @@ module Gem
   @ruby = nil
   @sources = []
 
+  @post_build_hooks     ||= []
   @post_install_hooks   ||= []
   @post_uninstall_hooks ||= []
   @pre_uninstall_hooks  ||= []
   @pre_install_hooks    ||= []
 
   ##
-  # Activates an installed gem matching +gem+.  The gem must satisfy
-  # +version_requirements+.
+  # Try to activate a gem containing +path+. Returns true if
+  # activation succeeded or wasn't needed because it was already
+  # activated. Returns false if it can't find the path in a gem.
+
+  def self.try_activate path
+    # finds the _latest_ version... regardless of loaded specs and their deps
+
+    # TODO: use find_all and bork if ambiguous
+
+    spec = Gem.searcher.find path
+    return false unless spec
+
+    begin
+      Gem.activate spec.name, "= #{spec.version}"
+    rescue Gem::LoadError # this could fail due to gem dep collisions, go lax
+      Gem.activate spec.name
+    end
+
+    return true
+  end
+
+  ##
+  # Activates an installed gem matching +dep+.  The gem must satisfy
+  # +requirements+.
   #
   # Returns true if the gem is activated, false if it is already
   # loaded, or an exception otherwise.
   #
-  # Gem#activate adds the library paths in +gem+ to $LOAD_PATH.  Before a Gem
+  # Gem#activate adds the library paths in +dep+ to $LOAD_PATH.  Before a Gem
   # is activated its required Gems are activated.  If the version information
   # is omitted, the highest version Gem of the supplied name is loaded.  If a
   # Gem is not found that meets the version requirements or a required Gem is
@@ -239,72 +231,83 @@ module Gem
   # More information on version requirements can be found in the
   # Gem::Requirement and Gem::Version documentation.
 
-  def self.activate(gem, *version_requirements)
-    if version_requirements.last.is_a?(Hash)
-      options = version_requirements.pop
+  def self.activate(dep, *requirements)
+    # TODO: remove options entirely
+    if requirements.last.is_a?(Hash)
+      options = requirements.pop
     else
       options = {}
     end
 
+    requirements = Gem::Requirement.default if requirements.empty?
+    dep = Gem::Dependency.new(dep, requirements) unless Gem::Dependency === dep
+
+    # TODO: remove sources entirely
     sources = options[:sources] || []
+    matches = Gem.source_index.search dep, true
+    report_activate_error(dep) if matches.empty?
 
-    if version_requirements.empty? then
-      version_requirements = Gem::Requirement.default
-    end
-
-    unless gem.respond_to?(:name) and
-           gem.respond_to?(:version_requirements) then
-      gem = Gem::Dependency.new(gem, version_requirements)
-    end
-
-    matches = Gem.source_index.find_name(gem.name, gem.version_requirements)
-    report_activate_error(gem) if matches.empty?
-
-    if @loaded_specs[gem.name] then
+    if @loaded_specs[dep.name] then
       # This gem is already loaded.  If the currently loaded gem is not in the
       # list of candidate gems, then we have a version conflict.
-      existing_spec = @loaded_specs[gem.name]
+      existing_spec = @loaded_specs[dep.name]
 
+      # TODO: unless dep.matches_spec? existing_spec then
       unless matches.any? { |spec| spec.version == existing_spec.version } then
-         sources_message = sources.map { |spec| spec.full_name }
-         stack_message = @loaded_stacks[gem.name].map { |spec| spec.full_name }
+        sources_message = sources.map { |spec| spec.full_name }
+        stack_message = @loaded_stacks[dep.name].map { |spec| spec.full_name }
 
-         msg = "can't activate #{gem} for #{sources_message.inspect}, "
-         msg << "already activated #{existing_spec.full_name} for "
-         msg << "#{stack_message.inspect}"
+        msg = "can't activate #{dep} for #{sources_message.inspect}, "
+        msg << "already activated #{existing_spec.full_name} for "
+        msg << "#{stack_message.inspect}"
 
-         e = Gem::LoadError.new msg
-         e.name = gem.name
-         e.version_requirement = gem.version_requirements
+        e = Gem::LoadError.new msg
+        e.name = dep.name
+        e.requirement = dep.requirement
 
-         raise e
+        raise e
       end
 
       return false
     end
 
-    # new load
     spec = matches.last
+
+    conf = spec.conflicts
+    unless conf.empty? then
+      why = conf.map { |act,con|
+        "#{act.full_name} conflicts with #{con.join(", ")}"
+      }.join ", "
+
+      # TODO: improve message by saying who activated `con`
+
+      raise LoadError, "Unable to activate #{spec.full_name}, because #{why}"
+    end
+
     return false if spec.loaded?
 
     spec.loaded = true
-    @loaded_specs[spec.name] = spec
+    @loaded_specs[spec.name]  = spec
     @loaded_stacks[spec.name] = sources.dup
 
-    # Load dependent gems first
-    spec.runtime_dependencies.each do |dep_gem|
-      activate dep_gem, :sources => [spec, *sources]
+    spec.runtime_dependencies.each do |spec_dep|
+      next if Gem.loaded_specs.include? spec_dep.name
+      specs = Gem.source_index.search spec_dep, true
+
+      if specs.size == 1 then
+        self.activate spec_dep
+      else
+        name = spec_dep.name
+        unresolved_deps[name] = unresolved_deps[name].merge spec_dep
+      end
     end
 
-    # bin directory must come before library directories
-    spec.require_paths.unshift spec.bindir if spec.bindir
+    unresolved_deps.delete spec.name
 
     require_paths = spec.require_paths.map do |path|
       File.join spec.full_gem_path, path
     end
 
-    sitelibdir = ConfigMap[:sitelibdir]
-
     # gem directories must come after -I and ENV['RUBYLIB']
     insert_index = load_path_insert_index
 
@@ -319,6 +322,10 @@ module Gem
     return true
   end
 
+  def self.unresolved_deps
+    @unresolved_deps ||= Hash.new { |h, n| h[n] = Gem::Dependency.new n }
+  end
+
   ##
   # An Array of all possible load paths for all versions of all gems in the
   # Gem installation.
@@ -351,7 +358,7 @@ module Gem
     requirements = Gem::Requirement.default if requirements.empty?
 
     unless gem.respond_to?(:name) and
-           gem.respond_to?(:version_requirements) then
+           gem.respond_to?(:requirement) then
       gem = Gem::Dependency.new gem, requirements
     end
 
@@ -361,26 +368,30 @@ module Gem
   ##
   # Find the full path to the executable for gem +name+.  If the +exec_name+
   # is not given, the gem's default_executable is chosen, otherwise the
-  # specifed executable's path is returned.  +version_requirements+ allows you
-  # to specify specific gem versions.
+  # specified executable's path is returned.  +requirements+ allows
+  # you to specify specific gem versions.
 
-  def self.bin_path(name, exec_name = nil, *version_requirements)
-    version_requirements = Gem::Requirement.default if
-      version_requirements.empty?
-    spec = Gem.source_index.find_name(name, version_requirements).last
+  def self.bin_path(name, exec_name = nil, *requirements)
+    requirements = Gem::Requirement.default if
+      requirements.empty?
+    specs = Gem.source_index.find_name(name, requirements)
 
     raise Gem::GemNotFoundException,
-          "can't find gem #{name} (#{version_requirements})" unless spec
+          "can't find gem #{name} (#{requirements})" if specs.empty?
+
+    specs = specs.find_all do |spec|
+      spec.executables.include?(exec_name)
+    end if exec_name
+
+    unless spec = specs.last
+      msg = "can't find gem #{name} (#{requirements}) with executable #{exec_name}"
+      raise Gem::GemNotFoundException, msg
+    end
 
     exec_name ||= spec.default_executable
 
     unless exec_name
-      msg = "no default executable for #{spec.full_name}"
-      raise Gem::Exception, msg
-    end
-
-    unless spec.executables.include? exec_name
-      msg = "can't find executable #{exec_name} for #{spec.full_name}"
+      msg = "no default executable for #{spec.full_name} and none given"
       raise Gem::Exception, msg
     end
 
@@ -391,7 +402,7 @@ module Gem
   # The mode needed to read a file as straight binary.
 
   def self.binary_mode
-    @binary_mode ||= RUBY_VERSION > '1.9' ? 'rb:ascii-8bit' : 'rb'
+    'rb'
   end
 
   ##
@@ -415,9 +426,7 @@ module Gem
 
     @@source_index = nil
 
-    MUTEX.synchronize do
-      @searcher = nil
-    end
+    @searcher = nil
   end
 
   ##
@@ -465,7 +474,7 @@ module Gem
 
   def self.dir
     @gem_home ||= nil
-    set_home(ENV['GEM_HOME'] || Gem.configuration.home || default_dir) unless @gem_home
+    set_home(ENV['GEM_HOME'] || default_dir) unless @gem_home
     @gem_home
   end
 
@@ -506,32 +515,37 @@ module Gem
   end
 
   ##
-  # Returns a list of paths matching +file+ that can be used by a gem to pick
+  # Returns a list of paths matching +glob+ that can be used by a gem to pick
   # up features from other gems.  For example:
   #
   #   Gem.find_files('rdoc/discover').each do |path| load path end
   #
-  # find_files search $LOAD_PATH for files as well as gems.
+  # if +check_load_path+ is true (the default), then find_files also searches
+  # $LOAD_PATH for files as well as gems.
   #
   # Note that find_files will return all files even if they are from different
   # versions of the same gem.
 
-  def self.find_files(path)
-    load_path_files = $LOAD_PATH.map do |load_path|
-      files = Dir["#{File.expand_path path, load_path}#{Gem.suffix_pattern}"]
+  def self.find_files(glob, check_load_path=true)
+    files = []
 
-      files.select do |load_path_file|
-        File.file? load_path_file.untaint
-      end
-    end.flatten
+    if check_load_path
+      files = $LOAD_PATH.map { |load_path|
+        Dir["#{File.expand_path glob, load_path}#{Gem.suffix_pattern}"]
+      }.flatten.select { |file| File.file? file.untaint }
+    end
 
-    specs = searcher.find_all path
+    specs = searcher.find_all glob
 
-    specs_files = specs.map do |spec|
-      searcher.matching_files spec, path
-    end.flatten
+    specs.each do |spec|
+      files.concat searcher.matching_files(spec, glob)
+    end
 
-    (load_path_files + specs_files).flatten.uniq
+    # $LOAD_PATH might contain duplicate entries or reference
+    # the spec dirs directly, so we prune.
+    files.uniq! if check_load_path
+
+    return files
   end
 
   ##
@@ -544,24 +558,26 @@ module Gem
   #   to be depending on HOME in those code samples. I propose that
   #   it should fallback to USERPROFILE and HOMEDRIVE + HOMEPATH (at
   #   least on Win32).
+  #++
 
   def self.find_home
-    ['HOME', 'USERPROFILE'].each do |homekey|
-      return ENV[homekey] if ENV[homekey]
-    end
-
-    if ENV['HOMEDRIVE'] && ENV['HOMEPATH'] then
-      return "#{ENV['HOMEDRIVE']}#{ENV['HOMEPATH']}"
-    end
-
-    begin
-      File.expand_path("~")
-    rescue
-      if File::ALT_SEPARATOR then
-          "C:/"
-      else
-          "/"
+    unless RUBY_VERSION > '1.9' then
+      ['HOME', 'USERPROFILE'].each do |homekey|
+        return File.expand_path(ENV[homekey]) if ENV[homekey]
       end
+
+      if ENV['HOMEDRIVE'] && ENV['HOMEPATH'] then
+        return File.expand_path("#{ENV['HOMEDRIVE']}#{ENV['HOMEPATH']}")
+      end
+    end
+
+    File.expand_path "~"
+  rescue
+    if File::ALT_SEPARATOR then
+      drive = ENV['HOMEDRIVE'] || ENV['SystemDrive']
+      File.join(drive.to_s, '/')
+    else
+      "/"
     end
   end
 
@@ -599,6 +615,20 @@ module Gem
     Zlib::Inflate.inflate data
   end
 
+  ##
+  # Get the default RubyGems API host. This is normally
+  # <tt>https://rubygems.org</tt>.
+
+  def self.host
+    @host ||= "https://rubygems.org"
+  end
+
+  ## Set the default RubyGems API host.
+
+  def self.host= host
+    @host = host
+  end
+
   ##
   # Return a list of all possible load paths for the latest version for all
   # gems in the Gem installation.
@@ -645,17 +675,29 @@ module Gem
   def self.load_path_insert_index
     index = $LOAD_PATH.index ConfigMap[:sitelibdir]
 
-    $LOAD_PATH.each_with_index do |path, i|
-      if path.instance_variables.include?(:@gem_prelude_index) or
-        path.instance_variables.include?('@gem_prelude_index') then
-        index = i
-        break
+    if QUICKLOADER_SUCKAGE then
+      $LOAD_PATH.each_with_index do |path, i|
+        if path.instance_variables.include?(:@gem_prelude_index) or
+            path.instance_variables.include?('@gem_prelude_index') then
+          index = i
+          break
+        end
       end
     end
 
     index
   end
 
+  ##
+  # Loads YAML, preferring Psych
+
+  def self.load_yaml
+    require 'psych'
+  rescue ::LoadError
+  ensure
+    require 'yaml'
+  end
+
   ##
   # The file name and line number of the caller of the caller of this method.
 
@@ -681,7 +723,7 @@ module Gem
     @gem_path ||= nil
 
     unless @gem_path then
-      paths = [ENV['GEM_PATH'] || Gem.configuration.path || default_path]
+      paths = [ENV['GEM_PATH'] || default_path]
 
       if defined?(APPLE_GEM_HOME) and not ENV['GEM_PATH'] then
         paths << APPLE_GEM_HOME
@@ -693,6 +735,27 @@ module Gem
     @gem_path
   end
 
+  ##
+  # Get the appropriate cache path.
+  #
+  # Pass a string to use a different base path, or nil/false (default) for
+  # Gem.dir.
+  #
+
+  def self.cache_dir(custom_dir=false)
+    File.join(custom_dir ? custom_dir : Gem.dir, 'cache')
+  end
+
+  ##
+  # Given a gem path, find the gem in cache.
+  #
+  # Pass a string as the second argument to use a different base path, or
+  # nil/false (default) for Gem.dir.
+
+  def self.cache_gem(filename, user_dir=false)
+    File.join(cache_dir(user_dir), filename)
+  end
+
   ##
   # Set array of platforms this RubyGems supports (primarily for testing).
 
@@ -711,6 +774,17 @@ module Gem
     @platforms
   end
 
+  ##
+  # Adds a post-build hook that will be passed an Gem::Installer instance
+  # when Gem::Installer#install is called.  The hook is called after the gem
+  # has been extracted and extensions have been built but before the
+  # executables or gemspec has been written.  If the hook returns +false+ then
+  # the gem's files will be removed and the install will be aborted.
+
+  def self.post_build(&hook)
+    @post_build_hooks << hook
+  end
+
   ##
   # Adds a post-install hook that will be passed an Gem::Installer instance
   # when Gem::Installer#install is called
@@ -730,7 +804,8 @@ module Gem
 
   ##
   # Adds a pre-install hook that will be passed an Gem::Installer instance
-  # when Gem::Installer#install is called
+  # when Gem::Installer#install is called.  If the hook returns +false+ then
+  # the install will be aborted.
 
   def self.pre_install(&hook)
     @pre_install_hooks << hook
@@ -746,17 +821,17 @@ module Gem
   end
 
   ##
-  # The directory prefix this RubyGems was installed at.
+  # The directory prefix this RubyGems was installed at. If your
+  # prefix is in a standard location (ie, rubygems is installed where
+  # you'd expect it to be), then prefix returns nil.
 
   def self.prefix
-    prefix = File.dirname File.expand_path(__FILE__)
+    prefix = File.dirname RUBYGEMS_DIR
 
-    if File.dirname(prefix) == File.expand_path(ConfigMap[:sitelibdir]) or
-       File.dirname(prefix) == File.expand_path(ConfigMap[:libdir]) or
-       'lib' != File.basename(prefix) then
-      nil
-    else
-      File.dirname prefix
+    if prefix != File.expand_path(ConfigMap[:sitelibdir]) and
+       prefix != File.expand_path(ConfigMap[:libdir]) and
+       'lib' == File.basename(RUBYGEMS_DIR) then
+      prefix
     end
   end
 
@@ -793,9 +868,7 @@ module Gem
   def self.refresh
     source_index.refresh!
 
-    MUTEX.synchronize do
-      @searcher = nil
-    end
+    @searcher = nil
   end
 
   ##
@@ -815,15 +888,15 @@ module Gem
 
     if matches.empty? then
       error = Gem::LoadError.new(
-          "Could not find RubyGem #{gem.name} (#{gem.version_requirements})\n")
+          "Could not find RubyGem #{gem.name} (#{gem.requirement})\n")
     else
       error = Gem::LoadError.new(
           "RubyGem version error: " +
-          "#{gem.name}(#{matches.first.version} not #{gem.version_requirements})\n")
+          "#{gem.name}(#{matches.first.version} not #{gem.requirement})\n")
     end
 
     error.name = gem.name
-    error.version_requirement = gem.version_requirements
+    error.requirement = gem.requirement
     raise error
   end
 
@@ -865,6 +938,29 @@ module Gem
     @ruby
   end
 
+  def self.latest_spec_for name
+    dependency  = Gem::Dependency.new name
+    fetcher     = Gem::SpecFetcher.fetcher
+    spec_tuples = fetcher.find_matching dependency
+
+    match = spec_tuples.select { |(n, _, p), _|
+      n == name and Gem::Platform.match p
+    }.sort_by { |(_, version, _), _|
+      version
+    }.last
+
+    match and fetcher.fetch_spec(*match)
+  end
+
+  def self.latest_version_for name
+    spec = latest_spec_for name
+    spec and spec.version
+  end
+
+  def self.latest_rubygems_version
+    latest_version_for "rubygems-update"
+  end
+
   ##
   # A Gem::Version for the currently running ruby.
 
@@ -885,18 +981,15 @@ module Gem
   # The GemPathSearcher object used to search for matching installed gems.
 
   def self.searcher
-    MUTEX.synchronize do
-      @searcher ||= Gem::GemPathSearcher.new
-    end
+    @searcher ||= Gem::GemPathSearcher.new
   end
 
   ##
   # Set the Gem home directory (as reported by Gem.dir).
 
   def self.set_home(home)
-    home = home.gsub(File::ALT_SEPARATOR, File::SEPARATOR) if File::ALT_SEPARATOR
+    home = home.gsub File::ALT_SEPARATOR, File::SEPARATOR if File::ALT_SEPARATOR
     @gem_home = home
-    ensure_gem_subdirectories(@gem_home)
   end
 
   private_class_method :set_home
@@ -921,18 +1014,6 @@ module Gem
     end
 
     @gem_path.uniq!
-    @gem_path.each do |path|
-      if 0 == File.expand_path(path).index(Gem.user_home)
-        next unless File.directory? Gem.user_home
-        unless win_platform? then
-          # only create by matching user
-          if Etc.getpwuid.nil? || Etc.getpwuid.uid != File::Stat.new(Gem.user_home).uid
-            next
-          end
-        end
-      end
-      ensure_gem_subdirectories path
-    end
   end
 
   private_class_method :set_paths
@@ -977,11 +1058,25 @@ module Gem
     @suffix_pattern ||= "{#{suffixes.join(',')}}"
   end
 
+  def self.loaded_path? path
+    # TODO: ruby needs a feature to let us query what's loaded in 1.8 and 1.9
+    $LOADED_FEATURES.find { |s|
+      s =~ /(^|\/)#{Regexp.escape path}#{Regexp.union(*Gem.suffixes)}$/
+    }
+  end
+
   ##
   # Suffixes for require-able paths.
 
   def self.suffixes
-    ['', '.rb', '.rbw', '.so', '.bundle', '.dll', '.sl', '.jar']
+    @suffixes ||= ['',
+                   '.rb',
+                   *%w(DLEXT DLEXT2).map { |key|
+                     val = RbConfig::CONFIG[key]
+                     next unless val and not val.empty?
+                     ".#{val}"
+                   }
+                  ].compact.uniq
   end
 
   ##
@@ -1037,6 +1132,51 @@ module Gem
     @@win_platform
   end
 
+  ##
+  # Load +plugins+ as ruby files
+
+  def self.load_plugin_files(plugins)
+    plugins.each do |plugin|
+
+      # Skip older versions of the GemCutter plugin: Its commands are in
+      # RubyGems proper now.
+
+      next if plugin =~ /gemcutter-0\.[0-3]/
+
+      begin
+        load plugin
+      rescue ::Exception => e
+        details = "#{plugin.inspect}: #{e.message} (#{e.class})"
+        warn "Error loading RubyGems plugin #{details}"
+      end
+    end
+  end
+
+  ##
+  # Find all 'rubygems_plugin' files in installed gems and load them
+
+  def self.load_plugins
+    load_plugin_files find_files('rubygems_plugin', false)
+  end
+
+  ##
+  # Find all 'rubygems_plugin' files in $LOAD_PATH and load them
+
+  def self.load_env_plugins
+    path = "rubygems_plugin"
+
+    files = []
+    $LOAD_PATH.each do |load_path|
+      globbed = Dir["#{File.expand_path path, load_path}#{Gem.suffix_pattern}"]
+
+      globbed.each do |load_path_file|
+        files << load_path_file if File.file?(load_path_file.untaint)
+      end
+    end
+
+    load_plugin_files files
+  end
+
   class << self
 
     ##
@@ -1044,6 +1184,12 @@ module Gem
 
     attr_reader :loaded_specs
 
+    ##
+    # The list of hooks to be run before Gem::Install#install finishes
+    # installation
+
+    attr_reader :post_build_hooks
+
     ##
     # The list of hooks to be run before Gem::Install#install does any work
 
@@ -1065,12 +1211,15 @@ module Gem
 
     attr_reader :pre_uninstall_hooks
 
-    # :stopdoc:
+  end
 
-    alias cache source_index # an alias for the old name
-
-    # :startdoc:
+  def self.cache # :nodoc:
+    warn "#{Gem.location_of_caller.join ':'}:Warning: " \
+      "Gem::cache is deprecated and will be removed on or after " \
+      "August 2011.  " \
+      "Use Gem::source_index."
 
+    source_index
   end
 
   ##
@@ -1078,64 +1227,104 @@ module Gem
 
   MARSHAL_SPEC_DIR = "quick/Marshal.#{Gem.marshal_version}/"
 
-  ##
-  # Location of legacy YAML quick gemspecs on remote repositories
+  autoload :Version, 'rubygems/version'
+  autoload :Requirement, 'rubygems/requirement'
+  autoload :Dependency, 'rubygems/dependency'
+  autoload :GemPathSearcher, 'rubygems/gem_path_searcher'
+  autoload :SpecFetcher, 'rubygems/spec_fetcher'
+  autoload :Specification, 'rubygems/specification'
+  autoload :Cache, 'rubygems/source_index'
+  autoload :SourceIndex, 'rubygems/source_index'
+  autoload :Platform, 'rubygems/platform'
+  autoload :Builder, 'rubygems/builder'
+  autoload :ConfigFile, 'rubygems/config_file'
+end
 
-  YAML_SPEC_DIR = 'quick/'
+module Kernel
+
+  remove_method :gem if 'method' == defined? gem # from gem_prelude.rb on 1.9
+
+  ##
+  # Use Kernel#gem to activate a specific version of +gem_name+.
+  #
+  # +requirements+ is a list of version requirements that the
+  # specified gem must match, most commonly "= example.version.number".  See
+  # Gem::Requirement for how to specify a version requirement.
+  #
+  # If you will be activating the latest version of a gem, there is no need to
+  # call Kernel#gem, Kernel#require will do the right thing for you.
+  #
+  # Kernel#gem returns true if the gem was activated, otherwise false.  If the
+  # gem could not be found, didn't match the version requirements, or a
+  # different version was already activated, an exception will be raised.
+  #
+  # Kernel#gem should be called *before* any require statements (otherwise
+  # RubyGems may load a conflicting library version).
+  #
+  # In older RubyGems versions, the environment variable GEM_SKIP could be
+  # used to skip activation of specified gems, for example to test out changes
+  # that haven't been installed yet.  Now RubyGems defers to -I and the
+  # RUBYLIB environment variable to skip activation of a gem.
+  #
+  # Example:
+  #
+  #   GEM_SKIP=libA:libB ruby -I../libA -I../libB ./mycode.rb
+
+  def gem(gem_name, *requirements) # :doc:
+    skip_list = (ENV['GEM_SKIP'] || "").split(/:/)
+    raise Gem::LoadError, "skipping #{gem_name}" if skip_list.include? gem_name
+    Gem.activate(gem_name, *requirements)
+  end
+
+  private :gem
 
 end
 
-module Config
-  # :stopdoc:
-  class << self
-    # Return the path to the data directory associated with the named
-    # package.  If the package is loaded as a gem, return the gem
-    # specific data directory.  Otherwise return a path to the share
-    # area as define by "#{ConfigMap[:datadir]}/#{package_name}".
-    def datadir(package_name)
-      Gem.datadir(package_name) ||
-        File.join(Gem::ConfigMap[:datadir], package_name)
-    end
-  end
-  # :startdoc:
+##
+# Return the path to the data directory associated with the named package.  If
+# the package is loaded as a gem, return the gem specific data directory.
+# Otherwise return a path to the share area as define by
+# "#{ConfigMap[:datadir]}/#{package_name}".
+
+def RbConfig.datadir(package_name)
+  warn "#{Gem.location_of_caller.join ':'}:Warning: " \
+    "RbConfig.datadir is deprecated and will be removed on or after " \
+    "August 2011.  " \
+    "Use Gem::datadir."
+
+  require 'rbconfig/datadir'
+
+  Gem.datadir(package_name) ||
+    File.join(Gem::ConfigMap[:datadir], package_name)
 end
 
 require 'rubygems/exceptions'
-require 'rubygems/version'
-require 'rubygems/requirement'
-require 'rubygems/dependency'
-require 'rubygems/gem_path_searcher'    # Needed for Kernel#gem
-require 'rubygems/source_index'         # Needed for Kernel#gem
-require 'rubygems/platform'
-require 'rubygems/builder'              # HACK: Needed for rake's package task.
 
-begin
-  require 'rubygems/defaults/operating_system'
-rescue LoadError
-end
-
-if defined?(RUBY_ENGINE) then
+gem_preluded = Gem::GEM_PRELUDE_SUCKAGE and defined? Gem
+unless gem_preluded then # TODO: remove guard after 1.9.2 dropped
   begin
-    require "rubygems/defaults/#{RUBY_ENGINE}"
+    ##
+    # Defaults the operating system (or packager) wants to provide for RubyGems.
+
+    require 'rubygems/defaults/operating_system'
   rescue LoadError
   end
+
+  if defined?(RUBY_ENGINE) then
+    begin
+      ##
+      # Defaults the ruby implementation wants to provide for RubyGems
+
+      require "rubygems/defaults/#{RUBY_ENGINE}"
+    rescue LoadError
+    end
+  end
 end
 
-require 'rubygems/config_file'
+##
+# Enables the require hook for RubyGems.
 
-if RUBY_VERSION < '1.9' then
-  require 'rubygems/custom_require'
-end
+require 'rubygems/custom_require'
 
 Gem.clear_paths
 
-plugins = Gem.find_files 'rubygems_plugin'
-
-plugins.each do |plugin|
-  begin
-    load plugin
-  rescue => e
-    warn "error loading #{plugin.inspect}: #{e.message} (#{e.class})"
-  end
-end
-
